/* Generate two 4-bit ALU portions for comp4 ALU
 * 1 May 2003: Michael Robinson */
#include <stdio.h>

int main()
{
   FILE *fplow, *fphigh;
   unsigned opcode, f, acc, mem, hres, lres, address, hf, lf;

   if( ( fplow = fopen( "alulow", "wt" ) ) == NULL )
   {
      perror( "genalu" );
      return -1;
   }
   if( ( fphigh = fopen( "aluhigh", "wt" ) ) == NULL )
   {
      perror( "genalu" );
      fclose( fplow );
      return -1;
   }

   /* Pinout of ALU chips:
    * Address (Input): 
    *   12 11 10  9  8  7  6  5  4  3  2  1  0
    *  +--+--+--+--+--+--+--+--+--+--+--+--+--+
    *  |   Acc     |   Mem     |F |  Opcode   |
    *  +--+--+--+--+--+--+--+--+--+--+--+--+--+
    *
    * Data (Output):
    *    7  6  5  4  3  2  1  0
    *  +--+--+--+--+--+--+--+--+
    *  | Unused | F|   Result  |
    *  +--+--+--+--+--+--+--+--+
    */

   for( opcode = 0; opcode < 16; opcode ++ )
   {
      for( acc = 0; acc < 16; acc ++ )
      {
         for( mem = 0; mem < 16; mem ++ )
         {
            for( f = 0; f < 2; f ++ )
            {
 address = 0xf & opcode | (( 0x1 & f ) << 4) | (( 0xf & mem ) << 5)
           | (( 0xf & acc ) << 9);
 switch( opcode )
 {
 case 0:  /* ACC */
   hres=acc;
   lres=acc;
   if( !(lres & 0xf) ) lf = 1; else lf = 0;
   if( !( hres & 0xf ) && f == 1 ) hf = 1; else hf = 0;
   break;
 case 1:  /* MEM */
   hres=mem;
   lres=mem;
   if( !(lres & 0xf) ) lf = 1; else lf = 0;
   if( !( hres & 0xf ) && f == 1 ) hf = 1; else hf = 0;
   break;
 case 2:  /* XOR */
   hres = mem ^ acc;
   lres = mem ^ acc;
   if( !(lres & 0xf) ) lf = 1; else lf = 0;
   if( !(hres & 0xf) && f == 1 ) hf = 1; else hf = 0; 
   break;
 case 3:  /* CLR */
   hres = 0;
   lres = 0;
   lf = 1;
   hf = 1;
   break;
 case 4:  /* NOA */
   hres = ~acc;
   lres = ~acc;
   if( !(lres & 0xf) ) lf = 1; else lf = 0;
   if( !(hres & 0xf) && f == 1 ) hf = 1; else hf = 0; 
   break;
 case 5:  /* NOM */
   hres = ~mem;
   lres = ~mem;
   if( !( lres & 0xf) ) lf = 1; else lf = 0;
   if( !( hres & 0xf ) && f == 1 ) hf = 1; else hf = 0; 
   break;
 case 6:  /* AND */
   hres = mem & acc;
   lres = mem & acc;
   if( !(lres & 0xf) ) lf = 1; else lf = 0;
   if( !( hres & 0xf ) && f == 1 ) hf = 1; else hf = 0; 
   break;
 case 7:  /* OR */ 
   hres = mem | acc;
   lres = mem | acc;
   if( !(lres & 0xf) ) lf = 1; else lf = 0;
   if( !( hres & 0xf ) && f == 1 ) hf = 1; else hf = 0; 
   break;
 case 8:  /* RLA */
   lres = ( acc << 1 ) | (0x1 & f);
   hres = ( acc << 1 ) | (0x1 & f);
   if( acc & 0x8 ) hf = 1; else hf = 0;
   if( acc & 0x8 ) lf = 1; else lf = 0;
   break;
 case 9:  /* RRA */
   lres = ( acc >> 1 ) | ((0x1 & f) << 3);
   hres = ( acc >> 1 ) | ((0x1 & f) << 3);
   if( acc & 0x1 ) hf = 1; else hf = 0;
   if( acc & 0x1 ) lf = 1; else lf = 0;
   break;
 case 10:  /* RLM */
   lres = ( mem << 1 ) | (0x1 & f);
   hres = ( mem << 1 ) | (0x1 & f);
   if( mem & 0x8 ) hf = 1; else hf = 0;
   if( mem & 0x8 ) lf = 1; else lf = 0;
   break;
 case 11:  /* RRM */
   lres = ( mem >> 1 ) | ((0x1 & f) << 3);
   hres = ( mem >> 1 ) | ((0x1 & f) << 3);
   if( mem & 0x1 ) hf = 1; else hf = 0;
   if( mem & 0x1 ) lf = 1; else lf = 0;
   break;
 case 12:  /* ADD */
   lres = acc + mem;
   hres = acc + mem + f;
   if( lres > 15 ) lf = 1; else lf = 0;
   if( hres > 15 ) hf = 1; else hf = 0;
   break;
 case 13: /* INA */
   lres = acc + 1;
   hres = acc + f;
   if( lres > 15 ) lf = 1; else lf = 0;
   if( hres > 15 ) hf = 1; else hf = 0;
   break;
 case 14: /* INM */
   lres = mem + 1;
   hres = mem + f;
   if( lres > 15 ) lf = 1; else lf = 0;
   if( hres > 15 ) hf = 1; else hf = 0;
   break;
 default:
   lres = 0;
   hres = 0;
   lf = 0;
   hf = 0;
   break;
 }
lres = ( lres & 0xf ) | ( (0x1 & lf) << 4 );
hres = ( hres & 0xf ) | ( (0x1 & hf) << 4 );

fprintf(fplow, "@%x\n%x\n", address, lres);
fprintf(fphigh, "@%x\n%x\n", address, hres);
            }
         }
      }
   } 

   fclose( fplow );
   fclose( fphigh );

   return 0;
}
